@@ -17,13 +17,16 @@ Contains the GdxExporter project item.
 """
 
 from copy import deepcopy
+from itertools import zip_longest
 import os.path
-from PySide2.QtCore import Slot
-from spinedb_api import clear_filter_configs
+from PySide2.QtCore import Slot, Qt
 from spinetoolbox.project_item.project_item import ProjectItem
-from spine_engine.utils.serialization import deserialize_path
+from spine_engine.utils.serialization import deserialize_path, serialize_url
+from spinedb_api import clear_filter_configs
 from spinedb_api.spine_io.exporters import gdx
-from ..utils import Database
+from ..commands import UpdateOutFileName
+from ..models import DatabaseListModel
+from ..utils import Database, exported_files_as_resources
 from ..item_base import ExporterBase
 from .commands import UpdateSettings
 from .executable_item import ExecutableItem
@@ -31,7 +34,9 @@ from .item_info import ItemInfo
 from .settings_pack import SettingsPack
 from .settings_state import SettingsState
 from .widgets.gdx_export_settings import GdxExportSettings
+from .widgets.export_list_item import ExportListItem
 from .worker import Worker
+from .exporter_notifications import ExporterNotifications
 
 
 class GdxExporter(ExporterBase):
@@ -67,8 +72,13 @@ class GdxExporter(ExporterBase):
             cancel_on_error (bool): True if execution should fail on all export errors,
                 False to ignore certain error cases; optional to provide backwards compatibility
         """
-        super().__init__(name, description, x, y, toolbox, project, databases, output_time_stamps, cancel_on_error)
+        super().__init__(name, description, x, y, toolbox, project, output_time_stamps, cancel_on_error)
+        self._notifications = ExporterNotifications()
         self._settings_pack = settings_pack if settings_pack is not None else SettingsPack()
+        if databases is None:
+            databases = list()
+        self._database_model = DatabaseListModel(databases)
+        self._inactive_databases = dict()
         self._settings_window = None
         self._worker = None
 
@@ -102,17 +112,172 @@ class GdxExporter(ExporterBase):
         return s
 
     def _check_notifications(self):
+        """See base class."""
         super()._check_notifications()
+        self._check_missing_file_names()
+        self._check_duplicate_file_names()
+        self._check_missing_specification()
+        self._report_notifications()
         self.add_notification(
             "GdxExporter is deprecated and will be removed in a future release. "
             "Please replace this item by general-purpose Exporter."
         )
 
+    def _report_notifications(self):
+        """Updates the exclamation icon and notifications labels."""
+        if self._icon is None:
+            return
+        self.clear_notifications()
+        if self._notifications.duplicate_output_file_name:
+            self.add_notification("Duplicate output file names.")
+        if self._notifications.missing_output_file_name:
+            self.add_notification("Output file name(s) missing.")
+        if self._notifications.missing_specification:
+            self.add_notification("Export specification missing.")
+
     def _check_missing_specification(self):
         self._notifications.missing_specification = (
             self._database_model.rowCount() != 0 and self._settings_pack.settings is None
         )
 
+    def _check_missing_file_names(self):
+        """Checks the status of output file names."""
+        self._notifications.missing_output_file_name = not all(
+            bool(db.output_file_name) for db in self._database_model.items()
+        )
+
+    def _check_duplicate_file_names(self):
+        """Checks for duplicate output file names."""
+        self._notifications.duplicate_output_file_name = False
+        names = set()
+        for db in self._database_model.items():
+            if db.output_file_name in names:
+                self._notifications.duplicate_output_file_name = True
+                break
+            names.add(db.output_file_name)
+
+    def database(self, url):
+        """
+        Returns database information for given URL.
+
+        Args:
+            url (str): database URL
+
+        Returns:
+            Database: database information
+        """
+        return self._database_model.item(url)
+
+    def _update_properties_tab(self):
+        """Updates the database list in the properties tab."""
+        database_list_storage = self._properties_ui.databases_list_layout
+        while not database_list_storage.isEmpty():
+            widget_to_remove = database_list_storage.takeAt(0)
+            widget_to_remove.widget().deleteLater()
+        self._export_list_items.clear()
+        for db in self._database_model.items():
+            item = self._export_list_items[db.url] = ExportListItem(db.url, db.output_file_name)
+            database_list_storage.addWidget(item)
+            item.file_name_changed.connect(self._update_out_file_name)
+        self._properties_ui.output_time_stamps_check_box.setCheckState(
+            Qt.Checked if self._append_output_time_stamps else Qt.Unchecked
+        )
+        self._properties_ui.cancel_on_error_check_box.setCheckState(
+            Qt.Checked if self._cancel_on_error else Qt.Unchecked
+        )
+
+    def upstream_resources_updated(self, resources):
+        """See base class."""
+        full_urls = set(r.url for r in resources if r.type_ == "database")
+        database_urls = set(clear_filter_configs(url) for url in full_urls)
+        old_urls = self._database_model.urls()
+        if database_urls != old_urls:
+            common = old_urls & database_urls
+            old_urls_by_base = dict()
+            for url in old_urls:
+                if url not in common:
+                    old_urls_by_base.setdefault(clear_filter_configs(url), list()).append(url)
+            old_urls_by_base = {base: sorted(url_list) for base, url_list in old_urls_by_base.items()}
+            new_urls_by_base = dict()
+            for url in database_urls:
+                if url not in common:
+                    new_urls_by_base.setdefault(clear_filter_configs(url), list()).append(url)
+            new_urls_by_base = {base: sorted(url_list) for base, url_list in new_urls_by_base.items()}
+            useless_oldies = set()
+            homeless_new_urls = set()
+            for new_base, newbies in new_urls_by_base.items():
+                oldies = old_urls_by_base.get(new_base)
+                if oldies is None:
+                    homeless_new_urls |= set(newbies)
+                    continue
+                for oldie, newbie in zip_longest(oldies, newbies):
+                    if oldie is None:
+                        homeless_new_urls.add(newbie)
+                        continue
+                    if newbie is None:
+                        useless_oldies.add(oldie)
+                        continue
+                    self._database_model.update_url(oldie, newbie)
+            useless_oldies |= old_urls - database_urls
+            for url in useless_oldies:
+                self._inactive_databases[url] = self._database_model.remove(url)
+            for url in homeless_new_urls:
+                db = self._inactive_databases.pop(url, None)
+                if db is None:
+                    db = Database()
+                    db.url = url
+                self._database_model.add(db)
+        self._full_url_model.set_urls(full_urls)
+        if self._active:
+            self._update_properties_tab()
+        self._check_notifications()
+
+    def replace_resources_from_upstream(self, old, new):
+        """See base class."""
+        for old_resource, new_resource in zip(old, new):
+            if not old_resource.type_ == "database":
+                continue
+            self._database_model.update_url(
+                clear_filter_configs(old_resource.url), clear_filter_configs(new_resource.url)
+            )
+            self._full_url_model.update_url(old_resource.url, new_resource.url)
+
+    def set_out_file_name(self, file_name, database_path):
+        """Updates the output file name for given database"""
+        if self._active:
+            export_list_item = self._export_list_items[database_path]
+            export_list_item.out_file_name_edit.setText(file_name)
+        database = self._database_model.item(database_path)
+        old_file_name = database.output_file_name
+        old_resources = self.resources_for_direct_successors() if old_file_name else []
+        database.output_file_name = file_name
+        self._notifications.missing_output_file_name = not file_name
+        self._check_duplicate_file_names()
+        self._report_notifications()
+        if self._exported_files is not None:
+            exported_files = self._exported_files.pop(old_file_name, None)
+            if exported_files is not None and file_name:
+                self._exported_files[file_name] = exported_files
+        if not old_file_name or not file_name:
+            self._resources_to_successors_changed()
+        else:
+            new_resources = self.resources_for_direct_successors()
+            olds = [resource for resource in old_resources if resource.label == old_file_name]
+            news = [resource for resource in new_resources if resource.label == file_name]
+            self._resources_to_successors_replaced(olds, news)
+
+    def resources_for_direct_successors(self):
+        """See base class."""
+        resources, self._exported_files = exported_files_as_resources(
+            self.name, self._exported_files, self.data_dir, self._database_model.items()
+        )
+        return resources
+
+    @Slot(str, str)
+    def _update_out_file_name(self, file_name, url):
+        """Pushes a new UpdateExporterOutFileNameCommand to the toolbox undo stack."""
+        self._toolbox.undo_stack.push(UpdateOutFileName(self, file_name, url))
+
     def _start_worker(self, database_url, update_settings=False):
         """Starts fetching settings using a worker in another thread."""
         worker = self._worker
@@ -250,6 +415,13 @@ class GdxExporter(ExporterBase):
     def item_dict(self):
         """Returns a dictionary corresponding to this item's configuration."""
         d = super().item_dict()
+        databases = list()
+        for db in self._database_model.items():
+            db_dict = db.to_dict()
+            serialized_url = serialize_url(db.url, self._project.project_dir)
+            db_dict["database_url"] = serialized_url
+            databases.append(db_dict)
+        d["databases"] = databases
         d["settings_pack"] = self._settings_pack.to_dict()
         return d
 
@@ -297,6 +469,7 @@ class GdxExporter(ExporterBase):
     def tear_down(self):
         """See base class."""
         super().tear_down()
+        self._database_model.deleteLater()
         if self._worker is not None:
             self._worker.thread.quit()
             self._worker.thread.wait()
