@@ -21,7 +21,12 @@ from PySide2.QtCore import QModelIndex, Qt, Slot
 from spinetoolbox.helpers import create_dir
 from spinetoolbox.project_item.project_item import ProjectItem
 from spinetoolbox.widgets.custom_menus import ItemSpecificationMenu
-from ..commands import UpdateCancelOnErrorCommand, ChangeItemSelectionCommand
+from ..commands import (
+    UpdateCancelOnErrorCommand,
+    ChangeItemSelectionCommand,
+    UpdatePurgeBeforeWritingCommand,
+    UpdateOnConflictCommand,
+)
 from ..models import CheckableFileListModel
 from .executable_item import ExecutableItem
 from .item_info import ItemInfo
@@ -38,6 +43,8 @@ class Importer(ProjectItem):
         project,
         specification_name="",
         cancel_on_error=True,
+        purge_before_writing=False,
+        on_conflict="merge",
         file_selection=None,
     ):
         """Importer class.
@@ -51,8 +58,9 @@ class Importer(ProjectItem):
             project (SpineToolboxProject): the project this item belongs to
             specification_name (str, optional): a spec name
             cancel_on_error (bool): if True the item's execution will stop on import error
+            purge_before_writing (bool): if True the item will purge target dbs before running
             file_selection (dict): a map from label to a bool indicating if the file item is checked
-       """
+        """
         super().__init__(name, description, x, y, project)
         # Make logs subdirectory for this item
         self._toolbox = toolbox
@@ -68,7 +76,9 @@ class Importer(ProjectItem):
                 f"Importer <b>{self.name}</b> should have a specification <b>{specification_name}</b> but it was not found"
             )
         self.cancel_on_error = cancel_on_error
-        self._file_model = CheckableFileListModel(header_label="Source files")
+        self._purge_before_writing = purge_before_writing
+        self.on_conflict = on_conflict
+        self._file_model = CheckableFileListModel(header_label="Available resources")
         self._file_model.set_initial_state(file_selection if file_selection is not None else dict())
         self._file_model.checked_state_changed.connect(self._push_file_selection_change_to_undo_stack)
 
@@ -91,7 +101,7 @@ class Importer(ProjectItem):
         """Notifies Toolbox of successful database import."""
         if execution_direction != "FORWARD":
             return
-        successors = [self._project.get_item(name) for name in self._project.successor_names(self)]
+        successors = [self._project.get_item(name) for name in self._project.successor_names(self.name)]
         committed_db_maps = set()
         for successor in successors:
             if successor.item_type() == "Data Store":
@@ -107,11 +117,14 @@ class Importer(ProjectItem):
         """Returns a dictionary of all shared signals and their handlers.
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
-        s[self._properties_ui.toolButton_open_dir.clicked] = lambda _: self.open_directory()
         s[self._properties_ui.toolButton_edit_specification.clicked] = self.edit_specification
         s[self._properties_ui.treeView_files.doubleClicked] = self._handle_files_double_clicked
         s[self._properties_ui.comboBox_specification.textActivated] = self._change_specification
         s[self._properties_ui.cancel_on_error_checkBox.stateChanged] = self._handle_cancel_on_error_changed
+        s[self._properties_ui.radioButton_on_conflict_merge.clicked] = self._update_on_conflict
+        s[self._properties_ui.radioButton_on_conflict_keep.clicked] = self._update_on_conflict
+        s[self._properties_ui.radioButton_on_conflict_replace.clicked] = self._update_on_conflict
+        s[self._properties_ui.checkBox_purge_before_writing.stateChanged] = self._handle_purge_before_writing_changed
         return s
 
     @Slot(str)
@@ -146,10 +159,69 @@ class Importer(ProjectItem):
         self._properties_ui.cancel_on_error_checkBox.setCheckState(check_state)
         self._properties_ui.cancel_on_error_checkBox.blockSignals(False)
 
+    @Slot(int)
+    def _handle_purge_before_writing_changed(self, _state):
+        purge_before_writing = self._properties_ui.checkBox_purge_before_writing.isChecked()
+        if self._purge_before_writing == purge_before_writing:
+            return
+        self._toolbox.undo_stack.push(UpdatePurgeBeforeWritingCommand(self, purge_before_writing))
+
+    def set_purge_before_writing(self, purge_before_writing):
+        """Sets purge_before_writing setting.
+
+        Args:
+            purge_before_writing (bool): purge_before_writing flag
+        """
+        self._purge_before_writing = purge_before_writing
+        if not self._active:
+            return
+        check_state = Qt.Checked if self._purge_before_writing else Qt.Unchecked
+        self._properties_ui.checkBox_purge_before_writing.blockSignals(True)
+        self._properties_ui.checkBox_purge_before_writing.setCheckState(check_state)
+        self._properties_ui.checkBox_purge_before_writing.blockSignals(False)
+
+    def _on_conflict(self):
+        """Reads the on_conflict strategy from UI."""
+        strategies = {
+            self._properties_ui.radioButton_on_conflict_merge: "merge",
+            self._properties_ui.radioButton_on_conflict_replace: "replace",
+            self._properties_ui.radioButton_on_conflict_keep: "keep",
+        }
+        return strategies[next(button for button in strategies if button.isChecked())]
+
+    def _set_on_conflict(self):
+        """Sets the on_conflict strategy in the UI."""
+        {
+            "merge": self._properties_ui.radioButton_on_conflict_merge,
+            "replace": self._properties_ui.radioButton_on_conflict_replace,
+            "keep": self._properties_ui.radioButton_on_conflict_keep,
+        }[self.on_conflict].setChecked(True)
+
+    @Slot(bool)
+    def _update_on_conflict(self, _checked):
+        on_conflict = self._on_conflict()
+        if self.on_conflict == on_conflict:
+            return
+        self._toolbox.undo_stack.push(UpdateOnConflictCommand(self, on_conflict))
+
+    def set_on_conflict(self, on_conflict):
+        """Sets on_conflict setting.
+
+        Args:
+            on_conflict (str): on_conflict
+        """
+        self.on_conflict = on_conflict
+        if not self._active:
+            return
+        self._set_on_conflict()
+
     def restore_selections(self):
         """Restores selections into shared widgets when this project item is selected."""
         self._properties_ui.cancel_on_error_checkBox.setCheckState(Qt.Checked if self.cancel_on_error else Qt.Unchecked)
-        self._properties_ui.label_name.setText(self.name)
+        self._properties_ui.checkBox_purge_before_writing.setCheckState(
+            Qt.Checked if self._purge_before_writing else Qt.Unchecked
+        )
+        self._set_on_conflict()
         self._properties_ui.treeView_files.setModel(self._file_model)
         self._update_ui()
 
@@ -167,10 +239,6 @@ class Importer(ProjectItem):
         """Saves selections in shared widgets for this project item into instance variables."""
         self._properties_ui.treeView_files.setModel(None)
 
-    def update_name_label(self):
-        """Update Importer properties tab name label. Used only when renaming project items."""
-        self._properties_ui.label_name.setText(self.name)
-
     def do_set_specification(self, specification):
         """see base class"""
         if not super().do_set_specification(specification):
@@ -198,15 +266,16 @@ class Importer(ProjectItem):
         filepath = None
         if index.isValid():
             resource = self._file_model.resource(index)
-            if resource is not None:
+            if resource.type_ == "database":
+                filepath = resource.url
+            else:
                 if not resource.hasfilepath:
                     self._logger.msg_error.emit("File does not exist yet.")
-                    filepath = None
                 else:
-                    filepath = resource.path
-                    if not os.path.exists(filepath):
+                    if not os.path.exists(resource.path):
                         self._logger.msg_error.emit(f"Cannot find file '{filepath}'.")
-                        filepath = None
+                    else:
+                        filepath = resource.path
         self._toolbox.show_specification_form(self.item_type(), self.specification(), self, filepath=filepath)
 
     def select_connector_type(self, index):
@@ -231,9 +300,14 @@ class Importer(ProjectItem):
         self._toolbox.undo_stack.push(ChangeItemSelectionCommand(self.name, self._file_model, index, checked))
 
     def upstream_resources_updated(self, resources):
-        self._file_model.update(r for r in resources if r.type_ != "database")
+        self._file_model.update(resources)
         self._check_notifications()
 
+    def replace_resources_from_upstream(self, old, new):
+        """See base class."""
+        for old_resource, new_resource in zip(old, new):
+            self._file_model.replace(old_resource, new_resource)
+
     def _check_notifications(self):
         self.clear_notifications()
         if not self.specification():
@@ -257,6 +331,8 @@ class Importer(ProjectItem):
         else:
             d["specification"] = self.specification().name
         d["cancel_on_error"] = self.cancel_on_error
+        d["purge_before_writing"] = self._purge_before_writing
+        d["on_conflict"] = self.on_conflict
         selections = list()
         for row in range(self._file_model.rowCount()):
             label, selected = self._file_model.checked_data(self._file_model.index(row, 0))
@@ -270,8 +346,22 @@ class Importer(ProjectItem):
         description, x, y = ProjectItem.parse_item_dict(item_dict)
         specification_name = item_dict.get("specification", "")
         cancel_on_error = item_dict.get("cancel_on_error", False)
+        purge_before_writing = item_dict.get("purge_before_writing", False)
+        on_conflict = item_dict.get("on_conflict", "merge")
         file_selection = {label: selected for label, selected in item_dict.get("file_selection", list())}
-        return Importer(name, description, x, y, toolbox, project, specification_name, cancel_on_error, file_selection)
+        return Importer(
+            name,
+            description,
+            x,
+            y,
+            toolbox,
+            project,
+            specification_name,
+            cancel_on_error,
+            purge_before_writing,
+            on_conflict,
+            file_selection,
+        )
 
     def notify_destination(self, source_item):
         """See base class."""
