@@ -21,24 +21,32 @@ import fnmatch
 import glob
 import os
 import os.path
+import sys
 import pathlib
 import shutil
 import time
 import uuid
+from contextlib import ExitStack
 from spine_engine.config import TOOL_OUTPUT_DIR
 from spine_engine.project_item.executable_item_base import ExecutableItemBase
 from spine_engine.spine_engine import ItemExecutionFinishState
-from spine_engine.utils.helpers import resolve_julia_executable, resolve_gams_executable
+from spine_engine.project_item.project_item_resource import (
+    cmd_line_arg_from_dict,
+    expand_cmd_line_args,
+    labelled_resource_args,
+)
+from spine_engine.utils.helpers import resolve_julia_executable, resolve_gams_executable, write_filter_id_file
 from .item_info import ItemInfo
-from .utils import file_paths_from_resources, find_file, flatten_file_path_duplicates, is_pattern
+from .utils import file_paths_from_resources, find_file, flatten_file_path_duplicates, is_pattern, make_dir_if_necessary
 from .output_resources import scan_for_resources
-from ..utils import cmd_line_arg_from_dict, expand_cmd_line_args, labelled_resource_args
 
 
 class ExecutableItem(ExecutableItemBase):
     """Tool project item's executable parts."""
 
-    def __init__(self, name, work_dir, tool_specification, cmd_line_args, options, project_dir, logger):
+    _MAX_RETRIES = 3
+
+    def __init__(self, name, work_dir, tool_specification, cmd_line_args, options, group_id, project_dir, logger):
         """
         Args:
             name (str): item's name
@@ -47,16 +55,18 @@ class ExecutableItem(ExecutableItemBase):
             tool_specification (ToolSpecification): a tool specification
             cmd_line_args (list): a list of command line argument to pass to the tool instance
             options (dict): misc tool options. See ``Tool`` for details.
+            group_id (str or None): execution group identifier
             project_dir (str): absolute path to project directory
             logger (LoggerInterface): a logger
         """
-        super().__init__(name, project_dir, logger)
+        super().__init__(name, project_dir, logger, group_id=group_id)
         self._work_dir = work_dir
         self._output_dir = str(pathlib.Path(self._data_dir, TOOL_OUTPUT_DIR))
         self._tool_specification = tool_specification
         self._cmd_line_args = cmd_line_args
         self._options = options
         self._tool_instance = None
+        self._retry_count = 0
 
     @property
     def options(self):
@@ -66,7 +76,7 @@ class ExecutableItem(ExecutableItemBase):
     def filter_id(self, filter_id):
         self._filter_id = filter_id
         self._logger.set_filter_id(filter_id)
-        filter_output_dir = os.path.join(self._output_dir, filter_id)
+        filter_output_dir = os.path.join(self._output_dir, self.hash_filter_id())
         try:
             os.makedirs(filter_output_dir, exist_ok=True)
             self._output_dir = filter_output_dir
@@ -98,6 +108,11 @@ class ExecutableItem(ExecutableItemBase):
             bool: True if the operation was successful, False otherwise
         """
         n_copied_files = 0
+        # Make sure that execution_dir (work dir) exists if there's something to copy.
+        # Needed by Executable tools running a shell command
+        if not make_dir_if_necessary(paths, execution_dir):
+            self._logger.msg_error.emit(f"Creating directory <b>{execution_dir}</b> failed")
+            return False
         for dst, src_path in paths.items():
             if not os.path.exists(src_path):
                 self._logger.msg_error.emit(f"\tFile <b>{src_path}</b> does not exist")
@@ -122,6 +137,10 @@ class ExecutableItem(ExecutableItemBase):
             try:
                 shutil.copyfile(src_path, dst_path)
                 n_copied_files += 1
+            except shutil.SameFileError:
+                # Happens in source dir exec mode when DC is passing a file as reference from the main program dir to
+                # source dir, which are equal
+                self._logger.msg_warning.emit("\tNo need to copy. File already available.")
             except OSError as e:
                 self._logger.msg_error.emit(f"Copying file <b>{src_path}</b> to <b>{dst_path}</b> failed")
                 self._logger.msg_error.emit(f"{e}")
@@ -142,19 +161,32 @@ class ExecutableItem(ExecutableItemBase):
         self._logger.msg.emit(f"\tCopied <b>{n_copied_files}</b> input file(s)")
         return True
 
-    def _copy_optional_input_files(self, paths):
+    def _copy_optional_input_files(self, paths, execution_dir):
         """
         Copies optional input files from given paths to work or source directory, depending on
         where the Tool specification requires them to be.
 
         Args:
             paths (dict): key is the source path, value is the destination path
+            execution_dir (str): Absolute path to work or source directory
+
+        Returns:
+            bool: False if creating the execution directory failed, True otherwise
         """
         n_copied_files = 0
+        # Make sure that execution_dir (work dir) exists if there's something to copy.
+        # Needed by Executable tools running a shell command
+        if not make_dir_if_necessary(paths, execution_dir):
+            self._logger.msg_error.emit(f"Creating directory <b>{execution_dir}</b> failed")
+            return False
         for src_path, dst_path in paths.items():
             try:
                 shutil.copyfile(src_path, dst_path)
                 n_copied_files += 1
+            except shutil.SameFileError:
+                # Happens in source dir exec mode when DC is passing a file as reference from the main program dir to
+                # source dir, which are equal
+                self._logger.msg_warning.emit("\tNo need to copy. File already available.")
             except OSError as e:
                 self._logger.msg_error.emit(f"Copying optional file <b>{src_path}</b> to <b>{dst_path}</b> failed")
                 self._logger.msg_error.emit(f"{e}")
@@ -172,6 +204,7 @@ class ExecutableItem(ExecutableItemBase):
                     )
                     self._logger.msg_warning.emit(msg)
         self._logger.msg.emit(f"\tCopied <b>{n_copied_files}</b> optional input file(s)")
+        return True
 
     def _copy_output_files(self, target_dir, execution_dir):
         """Copies Tool specification output files from work directory to given target directory.
@@ -314,6 +347,8 @@ class ExecutableItem(ExecutableItemBase):
         1. Tool has no specification
         2. Python or Julia Kernel spec not selected (jupyter kernel mode)
         3. Julia executable not set and not found in PATH (subprocess mode)
+        4. Trying to execute an Executable Tool Spec using a shell that is not supported
+        by the user's OS.
 
         Returns True otherwise.
         """
@@ -346,6 +381,18 @@ class ExecutableItem(ExecutableItemBase):
                     "Gams not found in PATH. Please select the Gams you want to use in Settings->Tools."
                 )
                 return False
+        elif self._tool_specification.tooltype.lower() == "executable":
+            if not self._tool_specification.main_prgm:
+                shell = self._tool_specification.execution_settings["shell"]
+                if sys.platform == "win32" and shell == "bash":
+                    self._logger.msg_error.emit("Bash shell is not supported on Windows. Please select another shell.")
+                    return False
+                if sys.platform != "win32" and (shell in ("cmd.exe", "powershell.exe")):
+                    self._logger.msg_error.emit(
+                        f"Selected shell is not supported on your platform [{sys.platform}]. "
+                        f"Please select another shell."
+                    )
+                    return False
         return True
 
     def execute(self, forward_resources, backward_resources):
@@ -405,22 +452,42 @@ class ExecutableItem(ExecutableItemBase):
             for k, v in optional_file_paths.items():
                 self._logger.msg.emit(f"\tFound <b>{len(v)}</b> files matching pattern <b>{k}</b>")
             optional_file_copy_paths = self._optional_output_destination_paths(optional_file_paths, execution_dir)
-            self._copy_optional_input_files(optional_file_copy_paths)
+            if not self._copy_optional_input_files(optional_file_copy_paths, execution_dir):
+                return False
         if not self._create_output_dirs(execution_dir):
             self._logger.msg_error.emit("Creating output subdirectories failed. Tool execution aborted.")
             return ItemExecutionFinishState.FAILURE
+        if not os.path.isdir(execution_dir):
+            self._logger.msg_warning.emit(
+                f"Work directory was not created because Tool Specification "
+                f"<b>{self._tool_specification.name}</b> does not contain any "
+                f"program files nor (optional) input files. Please <b>execute the "
+                f"Tool in source execution mode</b> or include files to the "
+                f"specification."
+            )
+            return ItemExecutionFinishState.FAILURE
         self._tool_instance = self._tool_specification.create_tool_instance(execution_dir, self._logger, self)
         resources = forward_resources + backward_resources
-        with labelled_resource_args(resources) as labelled_args:
+        with ExitStack() as stack:
+            labelled_args = labelled_resource_args(resources, stack)
             expanded_args = expand_cmd_line_args(self._cmd_line_args, labelled_args, self._logger)
             try:
                 self._tool_instance.prepare(expanded_args)
             except RuntimeError as error:
-                if str(error) != "":
+                if str(error):
                     self._logger.msg_error.emit(f"Failed to prepare Tool instance: {error}")
                 return ItemExecutionFinishState.FAILURE
             return_code = self._tool_instance.execute()
-        self._handle_output_files(return_code, execution_dir)
+            if return_code != 0 and self._tool_instance is not None and self._tool_instance.killed:
+                # NOTE: return_code will be 0 if the instance was killed by e.g. `exit(0)` in julia
+                # In this case we want to consider the tool successfull and not retry it
+                if self._retry_count < self._MAX_RETRIES:
+                    # Try again
+                    self._retry_count += 1
+                    self._logger.msg_warning.emit("The Tool process was found dead. Retrying...")
+                    return self.execute(forward_resources, backward_resources)
+                self._logger.msg_warning.emit("Maximum amount of retries reached.")
+        self._handle_output_files(return_code, self._filter_id, execution_dir)
         self._tool_instance = None
         # TODO: Check what return code is 'stopped' and return ItemExecutionFinishState.STOPPED in this case
         return ItemExecutionFinishState.SUCCESS if return_code == 0 else ItemExecutionFinishState.FAILURE
@@ -469,7 +536,7 @@ class ExecutableItem(ExecutableItemBase):
                 file_paths[file_path] = found_files
         return file_paths
 
-    def _handle_output_files(self, return_code, execution_dir):
+    def _handle_output_files(self, return_code, filter_id, execution_dir):
         """Copies Tool specification output files from work directory to result directory.
 
         Args:
@@ -494,6 +561,8 @@ class ExecutableItem(ExecutableItemBase):
         result_anchor = (
             f"<a style='color:#BB99FF;' title='{result_path}'" f"href='file:///{result_path}'>results directory</a>"
         )
+        if filter_id:
+            write_filter_id_file(filter_id, os.path.dirname(result_path))
         self._logger.msg.emit(f"*** Archiving output files to {result_anchor} ***")
         if self._tool_specification.outputfiles:
             saved_files, failed_files = self._copy_output_files(result_path, execution_dir)
@@ -587,7 +656,8 @@ class ExecutableItem(ExecutableItemBase):
         )
         cmd_line_args = [cmd_line_arg_from_dict(arg) for arg in item_dict["cmd_line_args"]]
         options = item_dict.get("options", {})
-        return cls(name, work_dir, specification, cmd_line_args, options, project_dir, logger)
+        group_id = item_dict.get("group_id")
+        return cls(name, work_dir, specification, cmd_line_args, options, group_id, project_dir, logger)
 
 
 def _count_files_and_dirs(paths):
@@ -612,7 +682,7 @@ def _count_files_and_dirs(paths):
 
 
 def _create_output_dir_timestamp():
-    """ Creates a new timestamp string that is used as Tool output
+    """Creates a new timestamp string that is used as Tool output
     directory.
 
     Returns:
@@ -632,7 +702,9 @@ def _execution_directory(work_dir, tool_specification):
     Returns the path to the execution directory, depending on ``execute_in_work``.
 
     If ``execute_in_work`` is ``True``, a new unique path will be returned.
-    Otherwise, the main program file path from tool specification is returned.
+    If a main program file does not exist, Tool spec definition file path is
+    returned. Otherwise, the main program file path from tool specification
+    is returned.
 
     Returns:
         str: a full path to next basedir
@@ -640,6 +712,8 @@ def _execution_directory(work_dir, tool_specification):
     if work_dir is not None:
         basedir = os.path.join(work_dir, _unique_dir_name(tool_specification))
         return basedir
+    if not tool_specification.path:
+        return tool_specification.default_execution_dir
     return tool_specification.path
 
 
