@@ -38,7 +38,6 @@ class ToolInstance:
 
     def __init__(self, tool_specification, basedir, settings, logger, owner):
         """
-
         Args:
             tool_specification (ToolSpecification): the tool specification for this instance
             basedir (str): the path to the directory where this instance should run
@@ -54,6 +53,13 @@ class ToolInstance:
         self.exec_mngr = None
         self.program = None  # Program to start in the subprocess
         self.args = list()  # List of command line arguments for the program
+        self.killed = False
+
+    def _update_killed(self):
+        try:
+            self.killed = self.exec_mngr.killed
+        except AttributeError:
+            pass
 
     @property
     def owner(self):
@@ -80,7 +86,6 @@ class ToolInstance:
 
         Args:
             args (list): Tool cmd line args
-            k_spec_dict (dict): Kernel spec name and related settings
         """
         raise NotImplementedError()
 
@@ -121,9 +126,13 @@ class GAMSToolInstance(ToolInstance):
         if ret != 0:
             try:
                 return_msg = self.tool_specification.return_codes[ret]
-                self._logger.msg_error.emit(f"\t<b>{return_msg}</b> [exit code:{ret}]")
+                self._logger.msg_error.emit(f"<b>{return_msg}</b> [exit code:{ret}]")
             except KeyError:
-                self._logger.msg_error.emit(f"\tUnknown return code ({ret})")
+                self._logger.msg_error.emit(f"Unknown return code ({ret})")
+            debug_anchor = self.tool_specification.make_debug_project(self.basedir)
+            if debug_anchor is not None:
+                self._logger.msg.emit(f"{debug_anchor}")
+        self._update_killed()
         self.exec_mngr = None
         return ret
 
@@ -149,19 +158,29 @@ class JuliaToolInstance(ToolInstance):
             self.exec_mngr = KernelExecutionManager(
                 self._logger, kernel_name, *self.args, group_id=self.owner.group_id, extra_switches=extra_switches
             )
-        else:
-            julia_exe = self._settings.value("appSettings/juliaPath", defaultValue="")
-            julia_exe = resolve_julia_executable(julia_exe)
-            julia_project_path = self._settings.value("appSettings/juliaProjectPath", defaultValue="")
-            self.program = [julia_exe]
-            self.program.append(f"--project={julia_project_path}")
+            return
+        julia_exe = self._settings.value("appSettings/juliaPath", defaultValue="")
+        julia_exe = resolve_julia_executable(julia_exe)
+        julia_project_path = self._settings.value("appSettings/juliaProjectPath", defaultValue="")
+        if use_julia_kernel == "1":
+            self.program = julia_exe
+            self.args = []
+            self.args.append(f"--project={julia_project_path}")
             if os.path.isfile(sysimage):
-                self.program.append(f"--sysimage={sysimage}")
-            alias = f"# Running 'julia {' '.join([self.tool_specification.main_prgm, *cmdline_args])}'"
-            self.exec_mngr = JuliaPersistentExecutionManager(
-                self._logger, self.program, self.args, alias, group_id=self.owner.group_id
-            )
-        # FIXME: script-less tools?
+                self.args.append(f"--sysimage={sysimage}")
+            if self.tool_specification.main_prgm:
+                self.args.append(self.tool_specification.main_prgm)
+            self.append_cmdline_args(args)
+            self.exec_mngr = ProcessExecutionManager(self._logger, self.program, *self.args, workdir=self.basedir)
+            return
+        self.program = [julia_exe]
+        self.program.append(f"--project={julia_project_path}")
+        if os.path.isfile(sysimage):
+            self.program.append(f"--sysimage={sysimage}")
+        alias = f"julia {' '.join([self.tool_specification.main_prgm, *cmdline_args])}"
+        self.exec_mngr = JuliaPersistentExecutionManager(
+            self._logger, self.program, self.args, alias, group_id=self.owner.group_id
+        )
 
     def execute(self):
         """Executes a prepared instance."""
@@ -172,6 +191,7 @@ class JuliaToolInstance(ToolInstance):
                 self._logger.msg_error.emit(f"\t<b>{return_msg}</b> [exit code:{ret}]")
             except KeyError:
                 self._logger.msg_error.emit(f"\tUnknown return code ({ret})")
+        self._update_killed()
         self.exec_mngr = None
         return ret
 
@@ -208,17 +228,30 @@ class PythonToolInstance(ToolInstance):
             fmt_cmdline_args = '["' + repr('", "'.join(cmdline_args)).strip("'") + '"]'
             self.args += [f"import sys; sys.argv = {fmt_cmdline_args};"]
             self.args += [f"import os; os.chdir({repr(self.basedir)})"]
-            exec_code = self._make_exec_code(fp, full_fp)
-            self.args += [exec_code]
-            alias = f"# Running 'python {' '.join([self.tool_specification.main_prgm, *cmdline_args[1:]])}'"
+            self.args += self._make_exec_code(fp, full_fp)
+            alias = f"python {' '.join([self.tool_specification.main_prgm, *cmdline_args[1:]])}"
             self.exec_mngr = PythonPersistentExecutionManager(
                 self._logger, self.program, self.args, alias, group_id=self.owner.group_id
             )
 
-    def _make_exec_code(self, fp, full_fp):
-        """Returns a string of code to run this tool instance in a python interactive session."""
-        glob = f'{{"__file__": "{full_fp}", "__name__": "__main__"}}'
-        return f"with open('{fp}', 'rb') as f: exec(compile(f.read(), '{fp}', 'exec'), {glob}, locals())"
+    @staticmethod
+    def _make_exec_code(fp, full_fp):
+        """Returns a list of lines of code to run this tool instance in a Python interactive session.
+
+        Args:
+            fp (str): relative path to main Python file
+            full_fp (str): absolute path to the main Python file
+
+        Returns:
+            list of str: lines of code
+        """
+        globals_dict = 'globals_dict = globals()'
+        update_globals_dict = f'globals_dict.update({{"__file__": "{full_fp}", "__name__": "__main__"}})'
+        compile_and_exec = (
+            f"with open('{fp}', 'rb') as f: exec(compile(f.read(), '{fp}', 'exec'), globals_dict, globals_dict)"
+            + os.linesep
+        )
+        return [globals_dict, update_globals_dict, compile_and_exec]
 
     def execute(self):
         """Executes a prepared instance."""
@@ -229,6 +262,7 @@ class PythonToolInstance(ToolInstance):
                 self._logger.msg_error.emit(f"\t<b>{return_msg}</b> [exit code:{ret}]")
             except KeyError:
                 self._logger.msg_error.emit(f"\tUnknown return code ({ret})")
+        self._update_killed()
         self.exec_mngr = None
         return ret
 
@@ -238,13 +272,38 @@ class ExecutableToolInstance(ToolInstance):
 
     def prepare(self, args):
         """See base class."""
-        batch_path = os.path.join(self.basedir, self.tool_specification.main_prgm)
-        if sys.platform != "win32":
-            self.program = "sh"
-            self.args.append(batch_path)
-        else:
-            self.program = batch_path
-        self.append_cmdline_args(args)
+        if not self.tool_specification.main_prgm:  # Run command
+            cmd = self.tool_specification.execution_settings["cmd"].split()  # Convert str to list
+            shell = self.tool_specification.execution_settings["shell"]
+            if not shell:
+                # If shell is not given (empty str), The first item in cmd list will be considered as self.program.
+                # The rest of the cmd list will be considered as cmd line args
+                self.program = cmd.pop(0)
+            else:
+                # If shell is given, the shell will be set as self.program and the cmd list will be considered
+                # as cmd line args
+                if shell == "bash":
+                    self.program = "sh"
+                else:
+                    self.program = shell
+            if self.program == "cmd.exe" or self.program == "cmd":
+                # If cmd.exe shell is not given the /C flag, it will just open cmd.exe in the Execution Log
+                if "/C" not in cmd:
+                    cmd = ["/C"] + cmd
+            # The final command line args list (self.args) consists of three (3) parts:
+            # 1. The first part is the cmd list
+            # 2. The second part is the Tool Specification cmd line args
+            # 3. The third part is the Tool cmd line args
+            self.args = cmd + self.tool_specification.cmdline_args + args
+        else:  # Run main program file
+            main_program_file = os.path.join(self.basedir, self.tool_specification.main_prgm)
+            if os.path.isfile(main_program_file):
+                if sys.platform != "win32":
+                    self.program = "sh"
+                    self.args.append(main_program_file)
+                else:
+                    self.program = main_program_file
+                self.append_cmdline_args(args)
         self.exec_mngr = ProcessExecutionManager(self._logger, self.program, *self.args, workdir=self.basedir)
 
     def execute(self):
@@ -256,5 +315,6 @@ class ExecutableToolInstance(ToolInstance):
                 self._logger.msg_error.emit(f"\t<b>{return_msg}</b> [exit code:{ret}]")
             except KeyError:
                 self._logger.msg_error.emit(f"\tUnknown return code ({ret})")
+        self._update_killed()
         self.exec_mngr = None
         return ret
